/*
    Copyright (c) 2015 digitalSTROM AG, Zurich, Switzerland

    Authors: Christian Hitz, digitalSTROM AG <christian.hitz@digitalstrom.com>
             Michael Tross, digitalSTROM AG <michael.tross@digitalstrom.com>
             Reto Flütsch, digitalSTROM AG <reto.fluetsch@digitalstrom.com>

    This file is part of vdSM.

    vdSM is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 2 of the License.


    vdSM is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vdSM. If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef DSM_H_
#define DSM_H_

#include "qp_port.h"
#include "types.h"
#include "output_channel.h"

/**
 * dSM signals
 */
enum DSMSignals {
    START_SIG = Q_USER_SIG,
    CONFIG_INIT_SIG,
    DATAMODEL_ZONE_ID_SIG,
    DS485_ADDRESS_SIG,
    SHORT_UPSTREAM_SIG,
    LONG_UPSTREAM_SIG,
    SYSTEM_TERM_SIG,
    SENSOR_PUSH_VALUE_SIG,
    VDC_UPSTREAM_SIG,
    MAX_PUBLISH_SIG,
    /* dS485 signals */
    DS485_RX_STARTUP_SIG,
    DS485_RX_JOIN_SIG,
    DS485_RX_DATA_SIG,
    DS485_TX_MSG_SIG,

    DS485_STOPPED_SIG,
    DS485_REQ_GET_NEXT_ACTIVE_BUS_MEMBER_SIG,
    DS485_REQ_GET_PREVIOUS_ACTIVE_BUS_MEMBER_SIG,
    DS485_RES_GET_BUS_MEMBER_BY_INDEX_SIG,
    DOWNSTREAM_ERROR_SIG,
    SELECT_B_SET_DSM_ID_SIG,
    SELECT_B_HELLO_SIG,
    SELECT_B_ANNOUNCE_DEVICE_SIG,
    SELECT_B_GET_DEVICE_STATISTICS_SIG,
    SELECT_B_GET_DEVICE_SENSOR_SIG,
    SELECT_B_GET_DEVICE_PARAMETER_SIG,
    SELECT_B_SET_DEVICE_PARAMETER_SIG,
    SELECT_B_FILTERED_LONG_UPSTREAM_SIG,
    SELECT_B_TEST_TRANSMISSION_QUALITY_SIG,
    SELECT_B_TEST_DEVICE_PRESENCE_SIG,
    SELECT_B_GET_DEVICE_DSLINK_SIG,
    SELECT_B_GET_ZONE_GROUP_DSLINK_SIG,
    SELECT_B_CONFIGURE_PING_SIG,
    SELECT_B_PING_INACTIVE_DEVICE_SIG,
    SELECT_B_STOP_SIG,
    SELECT_B_SELECT_SIG,
    SELECT_B_OUTPUT_CHANNEL_SIG,
    SELECT_B_SET_DEVICE_EXTRA_COMMAND,
    SELECT_B_SET_ZONE_GROUP_EXTRA_COMMAND,
    /* data model signals */
    DATAMODEL_REQ_MIN_SIG,
    DATAMODEL_REQ_ADD_DEVICE_SIG = DATAMODEL_REQ_MIN_SIG,
    DATAMODEL_REQ_ADD_GROUP_SIG,
    DATAMODEL_REQ_DEVICE_ENTRY_BY_SERIAL_NUMBER_SIG,
    DATAMODEL_REQ_DEVICE_ENTRY_BY_ID_SIG,
    DATAMODEL_REQ_DEVICE_INFO_OF_INDEX_SIG,
    DATAMODEL_REQ_DEVICE_SET_LOCKED_SIG,
    DATAMODEL_REQ_DEVICE_SET_BUTTON_ACTIVE_GROUP_SIG,
    DATAMODEL_REQ_DEVICE_SET_SET_LOCAL_PRIO_SIG,
    DATAMODEL_REQ_DEVICE_SET_NO_COMING_HOME_SIG,
    DATAMODEL_REQ_DEVICE_SET_BUTTON_APARTMENT_SIG,
    DATAMODEL_REQ_DEVICE_SET_BUTTON_OUTPUT_CHANNEL_SIG,
    DATAMODEL_REQ_DEVICE_SET_BUTTON_GLOBAL_GROUP_SIG,
    DATAMODEL_REQ_DEVICE_SET_NAME_SIG,
    DATAMODEL_REQ_DEVICE_SET_LTMODE_SIG,
    DATAMODEL_REQ_DEVICE_SET_OUTPUT_LOAD_SIG,
    DATAMODEL_REQ_DEVICE_SET_BUTTON_ID_AND_GROUP_SIG,
    DATAMODEL_REQ_DEVICE_SET_ZONE_SIG,
    DATAMODEL_REQ_DEVICE_PUSH_SENSOR_SIG,
    DATAMODEL_REQ_DEVICE_STATE_SIG,
    DATAMODEL_REQ_DSM_INFO_SIG,
    DATAMODEL_REQ_DSM_PROPERTIES_SIG,
    DATAMODEL_REQ_DSM_SET_APARTMENT_ID_SIG,
    DATAMODEL_REQ_DSM_SET_DSM_FLAGS_SIG,
    DATAMODEL_REQ_DSM_SET_NAME_SIG,
    DATAMODEL_REQ_DSM_SET_TEMPERATURE_LIMITS_SIG,
    DATAMODEL_REQ_DSM_SET_PING_INTERVALS_SIG,
    DATAMODEL_REQ_DSM_SET_UPSTREAM_FREQUENCY_SIG,
    DATAMODEL_REQ_FACTORY_RESET_SIG,
    DATAMODEL_REQ_GROUP_MEMBERSHIP_MODIFY_SIG,
    DATAMODEL_REQ_GROUP_SET_NAME_SIG,
    DATAMODEL_REQ_GROUP_SET_STOP_TIMEOUT_SIG,
    DATAMODEL_REQ_GROUP_SET_COLOR_SIG,
    DATAMODEL_REQ_PING_DEVICE_STATE_SIG,
    DATAMODEL_REQ_REMOVE_DEVICE_SIG,
    DATAMODEL_REQ_REMOVE_GROUP_SIG,
    DATAMODEL_REQ_REMOVE_INACTIVE_DEVICES_SIG,
    DATAMODEL_REQ_SCENE_INFO_SIG,
    DATAMODEL_REQ_SCENE_SET_NAME_SIG,
    DATAMODEL_REQ_SET_DSM_CONFIG_SIG,
    DATAMODEL_REQ_ZONE_COUNT_SIG,
    DATAMODEL_REQ_ZONE_DEVICE_COUNT_SIG,
    DATAMODEL_REQ_ZONE_GROUP_INFO_SIG,
    DATAMODEL_REQ_CLUSTER_INFO_SIG,
    DATAMODEL_REQ_CLUSTER_SET_NAME_SIG,
    DATAMODEL_REQ_CLUSTER_SET_STOP_TIMEOUT_SIG,
    DATAMODEL_REQ_CLUSTER_SET_COLOR_SIG,
    DATAMODEL_REQ_CLUSTER_SET_LOCATION_CLASS_SIG,
    DATAMODEL_REQ_CLUSTER_SET_CONFIG_LOCK_SIG,
    DATAMODEL_REQ_CLUSTER_SET_SCENE_LOCK_SIG,
    DATAMODEL_REQ_ZONE_INFO_BY_ZONE_ID_SIG,
    DATAMODEL_REQ_ZONE_INFO_OF_INDEX_SIG,
    DATAMODEL_REQ_ZONE_LOCAL_ID_SIG,
    DATAMODEL_REQ_ZONE_MODIFY_SIG,
    DATAMODEL_REQ_ZONE_SET_NAME_SIG,
    DATAMODEL_REQ_GET_HASH_SIG,
    DATAMODEL_REQ_MAX_SIG = DATAMODEL_REQ_GET_HASH_SIG,

    DATAMODEL_RES_ADD_DEVICE_SIG,
    DATAMODEL_RES_DEVICE_ENTRY_SIG,
    DATAMODEL_RES_DEVICE_STATE_SIG,
    DATAMODEL_RES_DSM_INFO_SIG,
    DATAMODEL_RES_DSM_PROPERTIES_SIG,
    DATAMODEL_RES_ERROR_SIG,
    DATAMODEL_RES_PING_DEVICE_SIG,
    DATAMODEL_RES_SCENE_INFO_SIG,
    DATAMODEL_RES_ZONE_COUNT_SIG,
    DATAMODEL_RES_ZONE_DEVICE_COUNT_SIG,
    DATAMODEL_RES_ZONE_GROUP_INFO_SIG,
    DATAMODEL_RES_CLUSTER_INFO_SIG,
    DATAMODEL_RES_ZONE_INFO_SIG,
    DATAMODEL_RES_ZONE_LOCAL_ID_SIG,
    DATAMODEL_RES_GET_HASH_SIG,
    DATAMODEL_STOPPED_SIG,
    DATAMODEL_CHANGED_SIG,
    DATAMODEL_RESET_MAX_TEMP_SIG,
    FLASH_DMA_DONE_SIG,
    FLASH_OP_TIMEOUT_SIG,

    DSMAPI_TIMEOUT_SIG,
    DSMAPI_REQUEST_TIMEOUT_SIG,
    CLICK_MIN_SIG,
    CT_TIP_1X_SIG = CLICK_MIN_SIG,
    CT_TIP_2X_SIG,
    CT_TIP_3X_SIG,
    CT_TIP_4X_SIG,
    CT_CLICK_1X_SIG,
    CT_CLICK_2X_SIG,
    CT_CLICK_3X_SIG,
    CT_2WAY_UP_TIP_1X_SIG,
    CT_2WAY_UP_TIP_2X_SIG,
    CT_2WAY_UP_TIP_3X_SIG,
    CT_2WAY_UP_TIP_4X_SIG,
    CT_2WAY_UP_HOLD_START_SIG,
    CT_2WAY_UP_HOLD_REPEAT_SIG,
    CT_2WAY_UP_HOLD_END_SIG,
    CT_2WAY_DOWN_TIP_1X_SIG,
    CT_2WAY_DOWN_TIP_2X_SIG,
    CT_2WAY_DOWN_TIP_3X_SIG,
    CT_2WAY_DOWN_CLICK_3X_SIG,
    CT_2WAY_DOWN_TIP_4X_SIG,
    CT_2WAY_DOWN_HOLD_START_SIG,
    CT_2WAY_DOWN_HOLD_REPEAT_SIG,
    CT_2WAY_DOWN_HOLD_END_SIG,
    CT_LOCAL_2WAY_DOWN_HOLD_START_SIG,
    CT_LOCAL_2WAY_DOWN_HOLD_REPEAT_SIG,
    CT_LOCAL_2WAY_DOWN_HOLD_END_SIG,
    CT_HOLD_START_SIG,
    CT_LOCAL_HOLD_START_SIG,
    CT_HOLD_END_SIG,
    CT_LOCAL_HOLD_END_SIG,
    CT_HOLD_REPEAT_SIG,
    CT_LOCAL_HOLD_REPEAT_SIG,
    CT_SHORT_LONG_SIG,
    CT_LOCAL_SHORT_LONG_SIG,
    CLICK_MAX_SIG = CT_LOCAL_SHORT_LONG_SIG,
    CT_LOCAL_OFF_SIG,
    CT_LOCAL_ON_SIG,
    CT_LOCAL_SHORT_SHORT_LONG_SIG,
    CT_LOCAL_STOP_SIG,

    CT_INVALID_SIG,
    CALL_INC_SIG,
    CALL_DEC_SIG,
    TIP_1X_ACTION_SIG,
    END_INC_SIG,
    END_DEC_SIG,
    END_LOCAL_PRIO_OFF_SIG,
    SET_SCENE_LOCAL_PRIO_SIG,
    STANDBY_SIG,
    END_STANDBY_SIG,
    DEEP_OFF_SIG,
    END_DEEP_OFF_SIG,
    ABSENT_SIG,
    PRESENT_SIG,
    END_ACCESS_SIG,
    SECURITY_ACTION_SIG,
    END_SECURITY_SIG,
    BLINDS_END_SIG,
    CALL_SCENE_BROADCAST_SIG,                   /**< broadcast call scene */
    CALL_SCENE_ZONE_GROUP_SIG,                  /**< call scene on a specific zone/group */
    INTERNAL_CLICK_FORWARD_SIG,
    DIRECT_SCENE_CALL_INC,
    CALL_ROOM_SCENE_SIG,
    CALL_ROOM_ON_SIG,
    CALL_ROOM_OFF_SIG,
    CALL_ROOM_STOP_SIG,
    BUTTON_GROUP_LOCAL_PRIO_OFF_SIG,
    BUTTON_GROUP_FORCE_SCENE_0_SIG,
    BUTTON_GROUP_STANDBY_SIG,
    BUTTON_GROUP_FORCE_SCENE_0_AND_STANDBY_SIG,
    BUTTON_GROUP_DEEP_OFF_SIG,
    CALL_SCENE_DEVICE_SIG,                      /**< call scene on a specific device */
    UNDO_SCENE_NUMBER_BROADCAST_SIG,            /**< broadcast undo scene number */
    UNDO_SCENE_NUMBER_ZONE_GROUP_SIG,           /**< undo scene number on a specific zone/group */
    UNDO_SCENE_NUMBER_DEVICE_SIG,               /**< undo scene number on a specific device */
    SAVE_SCENE_ZONE_SIG,
    SAVE_SCENE_DEVICE_SIG,
    UNDO_SCENE_ZONE_SIG,
    UNDO_SCENE_DEVICE_SIG,
    SET_OUTVAL_ZONE_GROUP_SIG,
    SET_OUTVAL_DEVICE_SIG,
    BLINK_ZONE_SIG,
    BLINK_DEVICE_SIG,
    SET_SCENE_LOCAL_PRIO_BROADCAST_SIG,
    REGISTER_SCENE_LOCAL_PRIO_BROADCAST_SIG,
    CALL_SCENE_MIN_BROADCAST_SIG,
    PROG_ON_DEVICE_SIG,
    PROG_ON_ZONE_SIG,
    PROG_OFF_DEVICE_SIG,
    PROG_OFF_ZONE_SIG,
    OPC_DIM_ZONE_GROUP_SIG,
    OPC_DIM_DEVICE_SIG,
    SELECT_ZONE_GROUP_SIG,
    ADD_DEVICE_TO_GROUP_SIG,
    REM_DEVICE_FROM_GROUP_SIG,
    REM_GROUP_FROM_ZONE_SIG,
    NOTIFY_USER_GROUP_SIG,
    RESET_DEVICE_SELECTION_SIG,
    LOCAL_STOP_SIG,
    SELECT_A_SHARED_TIMER_1_TIMEOUT_SIG,
    SELECT_A_SHARED_TIMER_2_TIMEOUT_SIG,
    SELECT_A_CLICK_TIMER_TIMEOUT_SIG,
    SELECT_A_STOP_TIMER_TIMEOUT_SIG,
    SELECT_A_REQ_ZONE_SCENE_HISTORY_SIG,
    SELECT_A_RES_ZONE_SCENE_HISTORY_SIG,
    SELECT_B_TIMEOUT_SIG,
    SELECT_B_PING_TIMEOUT_SIG,
    SELECT_B_PING_START_24_HOURS_PING_ROUND_SIG,
    SELECT_B_MINUTE_TIMEOUT_SIG,
    SELECT_B_TX_TIMEOUT_SIG,
    SELECT_B_DISABLE_JUMBLE_SIG,
    SELECT_A_REQ_ROOM_STATES_SIG,
    SELECT_A_RES_ROOM_STATES_SIG,
    /* MMI signals */
    MMI_TICK_SIG,
    MMI_SHUTDOWN_SIG,
    MMI_REQ_STATE_SIG,
    MMI_RES_STATE_SIG,
    MMI_USER_ACK_SIG,
    MMI_CONFIG_STATE_SIG,

    SELECT_B_PING_RESPONSE_SIG,

    HEATING_INITIALIZE_SIG,
    HEATING_MINUTE_TICK_SIG,
    HEATING_ZONE_MINUTE_TICK_SIG,
    HEATING_NOTIFY_CONFIG_UPDATE_SIG,
    HEATING_NOTIFY_CONFIG_CHANGE_SIG,
    HEATING_NOTIFY_SCENE_CHANGE_SIG,
    HEATING_FOLLOW_SENSOR_PUSH_SIG,
    HEATING_REQ_SET_STATE_SIG,

    VDC_TIMEOUT_SIG,
    VDC_ANNOUNCE_VDC_SIG,

    LAST_SIG,
};

/**
 * Short upstream event structure
 */
typedef struct {
    QEvent super;
    uint8_t quality;
    uint8_t flags;
    uint16_t source;
    uint8_t circuit;
    uint8_t sensor;
    uint8_t resend;
    union {
        struct {
            uint8_t key;
            uint8_t click;
        };
        struct {
            uint8_t eventType;
            uint8_t data;
            uint8_t index;
        };
    };
    uint8_t actionId;
    uint8_t actionMode;
} ShortUpstreamEvt;

/**
 * Long upstream event structure
 */
typedef struct {
    QEvent super;
    uint16_t source;
    uint8_t quality;
    uint8_t flags;
    uint32_t data;
    uint8_t circuit;
    uint8_t mainqualifier;
    uint8_t subqualifier;
    uint8_t errorCode;
} LongUpstreamEvt;

#define INIT_LongUpstreamEvt(evt_) (memset((void*)evt_ + offsetof(LongUpstreamEvt,source), 0, sizeof(LongUpstreamEvt) - offsetof(LongUpstreamEvt,source)))

/**
 * Downstream done event structure
 */
typedef struct {
    QEvent super;
    uint8_t transactionId;
    uint8_t padding;
} DownstreamDoneEvt;

/**
 * Front panel event structure
 */
typedef struct {
    QEvent super;
    uint8_t sreg;
    uint8_t padding;
} FrontPanelEvt;

/**
 * Scene basic event structure
 */
typedef struct {
    QEvent super;
    uint16_t sourceDeviceId;
    uint8_t sourceZoneId;
    uint8_t sourceGroupId;
    uint8_t sceneId;
    bool force;
    uint8_t outputChannel;
} SceneBasicEvt;

/**
 * Scene zone/group event structure
 */
typedef struct {
    SceneBasicEvt basic;
    uint8_t localZoneId;
    uint8_t groupId;
} SceneZoneGroupEvt;

/**
 * OPC dim zone/group event structure
 */
typedef struct {
    QEvent super;
    uint8_t localZoneId;
    uint8_t groupId;
    uint8_t channelId;
    OPCOperation operation;
} OPCDimZoneGroupEvt;

/**
 * OPC dim device event structure
 */
typedef struct {
    QEvent super;
    uint16_t deviceId;
    uint8_t channelId;
    OPCOperation operation;
} OPCDimDeviceEvt;

typedef struct {
    QEvent super;
    uint8_t localZoneId;
    uint8_t groupId;
    uint8_t event;
    uint8_t buttonId;
    uint16_t fid;
    uint8_t key;
    bool force;
    uint16_t deviceId;
    bool apartmentWide;
    uint8_t outputChannel;
} ClickForwardEvt;

/**
 * Scene device event structure
 */
typedef struct {
    QEvent super;
    uint16_t deviceId;
    uint8_t sceneId;
    bool force;
} SceneDeviceEvt;

typedef struct {
    QEvent super;
    uint8_t localZoneId;
    uint8_t groupId;
    uint8_t sceneId;
    uint8_t padding;
} ActionZoneGroupSceneEvt;

typedef struct {
    QEvent super;
    uint16_t deviceId;
    uint8_t sceneId;
    uint8_t padding;
} SaveSceneDeviceEvt;

typedef struct {
    QEvent super;
    uint8_t localZoneId;
    uint8_t groupId;
} ActionZoneEvt;

typedef struct {
    QEvent super;
    uint16_t zoneId;
    uint8_t groupId;
    uint8_t padding;
} ProgZoneEvt;

typedef struct {
    QEvent super;
    uint16_t deviceId;
} ActionDeviceEvt;

typedef struct {
    QEvent super;
    uint8_t localZoneId;
    uint8_t groupId;
    uint8_t outval;
    uint8_t padding;
} SetOutvalZoneGroupEvt;

typedef struct {
    QEvent super;
    uint16_t deviceId;
    uint8_t outval;
    uint8_t padding;
} SetOutvalDeviceEvt;

/**
 * Add device to group event structure
 */
typedef struct {
    QEvent super;
    uint16_t deviceId;
    uint8_t groupId;
    uint8_t padding;
} AddDeviceToGroupEvt;

/**
 * Remove device from group event structure
 */
typedef struct {
    QEvent super;
    uint16_t deviceId;
    uint8_t groupId;
    uint8_t padding;
} RemoveDeviceFromGroupEvt;

/**
 * Remove group from zone event structure
 */
typedef struct {
    QEvent super;
    uint8_t groupId;
    uint8_t localZoneId;
} RemoveGroupFromZoneEvt;

/**
 * Notify user group event structure
 */
typedef struct {
    QEvent super;
    uint8_t groupId;
    uint8_t stateMachineGroupId;
} NotifyUserGroupEvt;

enum {
    MAX_ZONE_SCENE_HISTORY    = 7,
};

/**
 * Get zone scene history request event structure
 */
typedef struct {
    QEvent super;
    QActive * requester;
    uint8_t localZoneId;
    uint8_t padding[3];
} ZoneSceneHistoryReqEvt;

/**
 * Get zone scene history response event structure
 */
typedef struct {
    QEvent super;
    uint8_t count;
    uint8_t groupId[MAX_ZONE_SCENE_HISTORY];
    uint8_t sceneId[MAX_ZONE_SCENE_HISTORY];
    uint8_t padding;
} ZoneSceneHistoryResEvt;

/**
 * Push sensor value
 */
typedef struct {
    QEvent super;
    uint16_t zoneId;
    uint8_t localZoneId;
    uint8_t groupId;
    uint8_t sensorType;
    uint8_t sensorPrecision;
    uint16_t sensorValue;
    bool isSourceExternal;
    uint8_t padding;
} ZoneGroupSensorPushEvt;

typedef struct {
    QEvent super;
    uint8_t localZoneId;
    uint8_t padding;
} ZoneStatesReqEvt;

typedef struct {
    QEvent super;
    uint16_t roomStates;
    uint16_t area1States;
    uint16_t area2States;
    uint16_t area3States;
    uint16_t area4States;
} ZoneStatesResEvt;

/**
 * Max macro
 */
#define min(x_, y_) (((x_) < (y_)) ? (x_) : (y_))

/**
 * Min macro
 */
#define max(x_, y_) (((x_) >= (y_)) ? (x_) : (y_))

/**
 * Swap bytes macro
 */
#define SWAP_BYTES(word_)   ((uint16_t)(((word_) >> 8) | ((word_) << 8)))

#endif /* DSM_H_ */
