/*
    Copyright (c) 2015 digitalSTROM AG, Zurich, Switzerland

    Authors: Christian Hitz, digitalSTROM AG <christian.hitz@digitalstrom.com>
             Michael Tross, digitalSTROM AG <michael.tross@digitalstrom.com>
             Reto Flütsch, digitalSTROM AG <reto.fluetsch@digitalstrom.com>

    This file is part of vdSM.

    vdSM is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 2 of the License.
    

    vdSM is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vdSM. If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef MMI_H_
#define MMI_H_

#include "qp_port.h"
#include "types.h"

/**
 * MMI line state enumeration
 */
typedef enum {
    MMI_LINE_IDLE               = 0,
    MMI_LINE_JUMBLING           = 1,
} MMILine;

/**
 * MMI dS485 state enumeration
 */
typedef enum {
    MMI_DS485_READY             = 0,
    MMI_DS485_INITIALIZING      = 1,
} MMIDS485;

/**
 * MMI configuration state enumeration
 */
typedef enum {
    MMI_CONFIG_OK               = 0,
    MMI_CONFIG_INVALID          = 1,
} MMIConfig;

/**
 * MMI temperature state enumeration
 */
typedef enum {
    MMI_TEMPERATURE_OK          = 0,
    MMI_TEMPERATURE_WARN        = 1,
    MMI_TEMPERATURE_ERROR       = 2,
} MMITemperature;

/**
 * MMI hardware state enumeration
 */
typedef enum {
    MMI_HARDWARE_OK             = 0,
    MMI_HARDWARE_UPDATING       = 1,
    MMI_HARDWARE_FAILURE        = 2,
    MMI_FLASH_FAILURE           = 3,
} MMIHardware;

/**
 * MMI line event structure
 */
typedef struct {
    QEvent super;
    MMILine line;
    uint8_t padding[3];
} MMILineStateEvt;

/**
 * MMI dS485 event structure
 */
typedef struct {
    QEvent super;
    MMIDS485 ds485;
    uint8_t padding[3];
} MMIDS485StateEvt;

/**
 * MMI configuration event structure
 */
typedef struct {
    QEvent super;
    MMIConfig config;
    uint8_t padding[3];
} MMIConfigStateEvt;

/**
 * MMI temperature event structure
 */
typedef struct {
    QEvent super;
    MMITemperature temperature;
    uint8_t padding[3];
} MMITemperatureStateEvt;

/**
 * MMI hardware event structure
 */
typedef struct {
    QEvent super;
    MMIHardware hardware;
    uint8_t padding[3];
} MMIHardwareStateEvt;

/**
 * MMI get state request event structure
 */
typedef struct {
    QEvent super;
    QActive * requester;
} MMIReqStateEvt;

/**
 * MMI state response event structure
 */
typedef struct {
    QEvent super;
    MMIHardware hardware;
    MMITemperature temperature;
    MMIConfig config;
    MMIDS485 ds485;
    MMILine line;
} MMIResStateEvt;

void MMI_ctor(void);

extern QActive * const AO_MMI; /* "opaque" pointer to MMI AO */

/**
 * Return margin of deferred event queue
 */
uint8_t MMI_GetDeferredQueueMargin(void);

#endif /* MMI_H_ */
